' SED_PBWIN - Use the PBWIN compiler
#COMPILE EXE
#DIM ALL
#INCLUDE "SDL.INC"
#INCLUDE "GLU.INC"
#INCLUDE "GDIPUTILS.INC"

$WindowCaption = "NeHe Lesson 16"

GLOBAL TextureHandles() AS DWORD
GLOBAL xrot AS SINGLE
GLOBAL yrot AS SINGLE
GLOBAL zoom AS SINGLE
GLOBAL filter AS LONG
GLOBAL xspeed AS SINGLE
GLOBAL yspeed AS SINGLE
GLOBAL fogfilter AS DWORD
GLOBAL fogMode() AS DWORD
GLOBAL fogColor() AS SINGLE

' =======================================================================================
' All the setup goes here
' =======================================================================================
SUB SetupScene ()

   LOCAL hr AS LONG
   LOCAL strTextureData AS STRING
   LOCAL TextureWidth, TextureHeight AS LONG
   DIM   LightAmbient(3) AS SINGLE
   DIM   LightDiffuse(3) AS SINGLE
   DIM   LightPosition(3) AS SINGLE
   DIM   fogMode(2) AS DWORD
   DIM   fogColor(3) AS SINGLE

   ' Storage for three types of fog
   fogMode(0) = %GL_EXP
   fogMode(1) = %GL_EXP2
   fogMode(2) = %GL_LINEAR

   ' Fog color
   fogColor(0) = 0.5!
   fogColor(1) = 0.5!
   fogColor(2) = 0.5!
   fogColor(3) = 1.0!

   REDIM TextureHandles(2) AS DWORD

   ' Load bitmap texture from disk
   hr = GdiPlusLoadTexture("crate.bmp", TextureWidth, TextureHeight, strTextureData, %TRUE)

   ' Assign an OpenGL handle to this texture
   glGenTextures 3, TextureHandles(0)

   ' Activate our newly created texture
   glEnable %GL_TEXTURE_2D

   ' Create Nearest Filtered Texture
   glBindTexture %GL_TEXTURE_2D, TextureHandles(0)
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_NEAREST
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_NEAREST
   glTexImage2D %GL_TEXTURE_2D, 0, 3, TextureWidth, TextureHeight, 0, _
                %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)

   ' Create Linear Filtered Texture
   glBindTexture %GL_TEXTURE_2D, TextureHandles(1)
   glTexParameteri %GL_TEXTURE_2D,%GL_TEXTURE_MAG_FILTER,%GL_LINEAR
   glTexParameteri %GL_TEXTURE_2D,%GL_TEXTURE_MIN_FILTER,%GL_LINEAR
   glTexImage2D %GL_TEXTURE_2D, 0, 3, TextureWidth, TextureHeight, 0, _
                %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)


   ' Create MipMapped Texture
   glBindTexture %GL_TEXTURE_2D, TextureHandles(2)
   glTexParameteri %GL_TEXTURE_2D,%GL_TEXTURE_MAG_FILTER,%GL_LINEAR
   glTexParameteri %GL_TEXTURE_2D,%GL_TEXTURE_MIN_FILTER,%GL_LINEAR_MIPMAP_NEAREST
   gluBuild2DMipmaps %GL_TEXTURE_2D, 3, TextureWidth, TextureHeight, _
                 %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)

   glEnable %GL_TEXTURE_2D

   ' Specify clear values for the color buffers
   glClearColor 0.5!, 0.5!, 0.5!, 1.0!
   ' Specify the clear value for the depth buffer
   glClearDepth 1.0!
   ' Specify the value used for depth-buffer comparisons
   glDepthFunc %GL_LESS
   ' Enable depth comparisons and update the depth buffer
   glEnable %GL_DEPTH_TEST
   ' Select smooth shading
   glShadeModel %GL_SMOOTH

   ARRAY ASSIGN LightAmbient()  = 0.5!, 0.5!, 0.5!, 1.0!
   ARRAY ASSIGN LightDiffuse()  = 1.0!, 1.0!, 1.0!, 1.0!
   ARRAY ASSIGN LightPosition() = 0.0!, 0.0!, 2.0!, 1.0!

   glLightfv %GL_LIGHT1, %GL_AMBIENT, LightAmbient(0)
   glLightfv %GL_LIGHT1, %GL_DIFFUSE, LightDiffuse(0)
   glLightfv %GL_LIGHT1, %GL_POSITION, LightPosition(0)
   glEnable %GL_LIGHT1

   glFogi %GL_FOG_MODE, fogMode(fogfilter)  ' Fog mode
   glFogfv %GL_FOG_COLOR, fogColor(0)       ' Set fog color
   glFogf %GL_FOG_DENSITY, 0.35!            ' How dense will the fog be
   glHint %GL_FOG_HINT, %GL_DONT_CARE       ' Fog hint value
   glFogf %GL_FOG_START, 1.0!               ' Fog start depth
   glFogf %GL_FOG_END, 5.0!                 ' Fog end depth
   glEnable %GL_FOG                         ' Enables GL_FOG

   zoom = -5.0

END SUB
' =======================================================================================

' =======================================================================================
' Draw the scene
' =======================================================================================
SUB DrawScene (BYVAL nWidth AS LONG, BYVAL nHeight AS LONG)

   LOCAL x_m, y_m, z_m, u_m, v_m AS SINGLE
   LOCAL loop_m AS LONG
   LOCAL xtrans, ztrans, ytrans AS SINGLE
   LOCAL sceneroty AS SINGLE

   ' Clear the screen buffer
   glClear %GL_COLOR_BUFFER_BIT OR %GL_DEPTH_BUFFER_BIT

   ' Select and setup the projection matrix
   glMatrixMode %GL_PROJECTION
   glLoadIdentity

   ' Prevent divide by zero making height equal one
   IF nHeight = 0 THEN nHeight = 1

   ' Calculate the aspect ratio of the window
   gluPerspective 45.0!, nWidth/nHeight, 1.0!, 100.0!

   ' Select and setup the modelview matrix
   glMatrixMode %GL_MODELVIEW
   glLoadIdentity

   glTranslatef 0.0!, 0.0!, zoom

   glRotatef xrot, 1.0!, 0.0!, 0.0!
   glRotatef yrot, 0.0!, 1.0!, 0.0!

   glBindTexture %GL_TEXTURE_2D, TextureHandles(filter)

   glBegin %GL_QUADS
      ' Front Face
      glNormal3f   0.0!, 0.0!, 1.0
      glTexCoord2f 0.0!, 0.0! : glVertex3f -1.0!, -1.0!,  1.0!
      glTexCoord2f 1.0!, 0.0! : glVertex3f  1.0!, -1.0!,  1.0!
      glTexCoord2f 1.0!, 1.0! : glVertex3f  1.0!,  1.0!,  1.0!
      glTexCoord2f 0.0!, 1.0! : glVertex3f -1.0!,  1.0!,  1.0!
      ' Back Face
      glNormal3f   0.0!, 0.0!, -1.0!
      glTexCoord2f 1.0!, 0.0! : glVertex3f -1.0!, -1.0!, -1.0!
      glTexCoord2f 1.0!, 1.0! : glVertex3f -1.0!,  1.0!, -1.0!
      glTexCoord2f 0.0!, 1.0! : glVertex3f  1.0!,  1.0!, -1.0!
      glTexCoord2f 0.0!, 0.0! : glVertex3f  1.0!, -1.0!, -1.0!
      ' Top Face
      glNormal3f   0.0!, 1.0!, 0.0!
      glTexCoord2f 0.0!, 1.0! : glVertex3f -1.0!,  1.0!, -1.0!
      glTexCoord2f 0.0!, 0.0! : glVertex3f -1.0!,  1.0!,  1.0!
      glTexCoord2f 1.0!, 0.0! : glVertex3f  1.0!,  1.0!,  1.0!
      glTexCoord2f 1.0!, 1.0! : glVertex3f  1.0!,  1.0!, -1.0!
      ' Bottom Face
      glNormal3f   0.0!,-1.0!, 0.0!
      glTexCoord2f 1.0!, 1.0! : glVertex3f -1.0!, -1.0!, -1.0!
      glTexCoord2f 0.0!, 1.0! : glVertex3f  1.0!, -1.0!, -1.0!
      glTexCoord2f 0.0!, 0.0! : glVertex3f  1.0!, -1.0!,  1.0!
      glTexCoord2f 1.0!, 0.0! : glVertex3f -1.0!, -1.0!,  1.0!
      ' Right face
      glNormal3f   1.0!, 0.0!, 0.0!
      glTexCoord2f 1.0!, 0.0! : glVertex3f  1.0!, -1.0!, -1.0!
      glTexCoord2f 1.0!, 1.0! : glVertex3f  1.0!,  1.0!, -1.0!
      glTexCoord2f 0.0!, 1.0! : glVertex3f  1.0!,  1.0!,  1.0!
      glTexCoord2f 0.0!, 0.0! : glVertex3f  1.0!, -1.0!,  1.0!
      ' Left Face
      glNormal3f  -1.0!,  0.0!, 0.0!
      glTexCoord2f 0.0!, 0.0! : glVertex3f -1.0!, -1.0!, -1.0!
      glTexCoord2f 1.0!, 0.0! : glVertex3f -1.0!, -1.0!,  1.0!
      glTexCoord2f 1.0!, 1.0! : glVertex3f -1.0!,  1.0!,  1.0!
      glTexCoord2f 0.0!, 1.0! : glVertex3f -1.0!,  1.0!, -1.0!
   glEnd

   xrot = xrot + xspeed
   yrot = yrot + yspeed

   ' // Swap buffers
   SDL_GL_SwapBuffers

END SUB
' =======================================================================================

' =======================================================================================
' Main
' =======================================================================================
FUNCTION PBMAIN () AS LONG

   LOCAL pscreen AS SDL_Surface PTR
   LOCAL nWidth, nHeight, bpp AS LONG
   LOCAL lResult, bFullScreen AS LONG
   LOCAL szCaption AS ASCIIZ * 256
   LOCAL dwFlags AS DWORD
   LOCAL t, t0, fps AS DOUBLE
   LOCAL nFrames AS LONG
   LOCAL lp, fp, gp, light AS LONG

   szCaption = $WindowCaption   ' Window caption

	' Ask the user which screen mode he prefers
	lResult = MessageBox(%NULL, "Would you like to run in fullscreen mode?", _
             "Start fullScreen?", %MB_YESNOCANCEL OR %MB_ICONQUESTION)
   SELECT CASE lResult
      CASE %IDCANCEL : EXIT FUNCTION
      CASE %IDYES    : bFullScreen = %TRUE
      CASE %IDNO     : bFullScreen = %FALSE
   END SELECT

   ' // Initialize SDL's subsystems - in this case, only video.
   IF SDL_Init(%SDL_INIT_VIDEO) < 0 THEN
      MSGBOX "Unable to init SDL: " & SDL_GetError()
      EXIT FUNCTION
   END IF

   ' // Attempt to create a 640x480 window
   nWidth = 640 : nHeight = 480 : bpp = 16
   IF bFullScreen THEN
      dwFlags = %SDL_OPENGL OR %SDL_FULLSCREEN
   ELSE
      dwFlags = %SDL_OPENGL OR %SDL_RESIZABLE
   END IF
   pscreen = SDL_SetVideoMode(nWidth, nHeight, bpp, dwFlags)
   SDL_WM_SetCaption szCaption, ""

   ' // If we fail, return error.
   IF pscreen = %NULL THEN
      SDL_Quit
      MSGBOX "Unable to set the video mode: " & SDL_GetError()
      EXIT FUNCTION
   END IF

   ' // Set up the scene
   SetupScene

   ' Enable keyboard repeat
   SDL_EnableKeyRepeat %SDL_DEFAULT_REPEAT_DELAY, %SDL_DEFAULT_REPEAT_INTERVAL

   ' // Main loop
   LOCAL done AS LONG
   LOCAL uevent AS SDL_Event
   WHILE done = %FALSE

      ' // Poll for events, and handle the ones we care about.
      WHILE SDL_PollEvent(VARPTR(uevent))
         SELECT CASE uevent.type
            CASE %SDL_KEYDOWN
               SELECT CASE uevent.key.keysym.sym
                  CASE %SDLK_L
                     IF ISFALSE lp THEN
                        lp = %TRUE
                        light = NOT light
                        IF ISFALSE light THEN
                           glDisable %GL_LIGHTING
                        ELSE
                           glEnable %GL_LIGHTING
                        END IF
                     END IF
                  CASE %SDLK_F
                     IF ISFALSE fp THEN
                        fp = %TRUE
                        filter = filter + 1
                        IF filter > 2 THEN filter = 0
                     END IF
                  CASE %SDLK_G
                     IF ISFALSE gp THEN
                        gp = %TRUE
                        fogfilter = fogfilter + 1
                        IF fogfilter > 2 THEN fogfilter = 0
                        glFogi %GL_FOG_MODE, fogMode(fogfilter)  ' Fog mode
                     END IF
                  CASE %SDLK_PAGEUP
                     zoom = zoom - 0.02!
                  CASE %SDLK_PAGEDOWN
                     zoom = zoom + 0.02!
                  CASE %SDLK_UP
                     xspeed = xspeed - 0.01!
                  CASE %SDLK_DOWN
                     xspeed = xspeed + 0.01!
                  CASE %SDLK_LEFT
                     yspeed = yspeed - 0.01!
                  CASE %SDLK_RIGHT
                     yspeed = yspeed + 0.01!
               END SELECT
            CASE %SDL_KEYUP
               ' // Quit if escape is pressed
               IF uevent.key.keysym.sym = %SDLK_ESCAPE THEN
                  done = %TRUE
                  EXIT LOOP
               END IF
               ' Reset flag
               gp = %FALSE
               fp = %FALSE
               lp = %FALSE
            CASE %SDL_VIDEORESIZE
               ' // Change the video mode to the new window size
               nWidth = uevent.resize.w
               nHeight = uevent.resize.h
               pscreen = SDL_SetVideoMode(nWidth, nHeight, bpp, dwFlags)
               glDeleteTextures(3, TextureHandles(0))
               SetupScene
            CASE %SDL_QUIT
               done = %TRUE
               EXIT LOOP
         END SELECT
      WEND

      IF ISFALSE bFullScreen THEN
         ' Get time and mouse position
         t = INT(TIMER)
         ' Calculate and display FPS (frames per second)
         IF t > t0 OR nFrames = 0 THEN
            fps = nFrames \ (t - t0)
            wsprintf szCaption, $WindowCaption & " (%i FPS)", BYVAL fps
            SDL_WM_SetCaption szCaption, szCaption
            t0 = t
            nFrames = 0
         END IF
         nFrames = nFrames + 1
      END IF

      ' // Draw the scene
      DrawScene nWidth, nHeight

   WEND

   ' Delete the textures
   glDeleteTextures(3, TextureHandles(0))

   ' Shut down SDL
   SDL_Quit

END FUNCTION
' =======================================================================================
